@@ -29,6 +29,8 @@
 #include "Data.h"
 #include "Pointer.h"
 
+#include <boost/shared_ptr.hpp>
+
 //#include "dataStructureGroups.h"
 
 #include "DataStructurePluginManager.h"
@@ -42,36 +44,41 @@ public:
     QString _buf;
     QString _lastSavedDocumentPath;
     QString _name;
-    qreal _xLeft;
-    qreal _xRight;
-    qreal _yTop;
-    qreal _yBottom;
+    qreal _left;
+    qreal _right;
+    qreal _top;
+    qreal _bottom;
     qreal _minWidth;
     qreal _minHeight;
     bool _modified;
     bool _saved;
-    DataStructure *_activeDataStructure;
+    DataStructurePtr _activeDataStructure;
     QPointer<DataStructurePluginInterface> _dataStructureType;
     QtScriptBackend* _engineBackend;
-    QList<DataStructure*> _dataStructures;
+    QList< DataStructurePtr > _dataStructures;
 
 
 };
 
-Document::Document(const QString& name, qreal xLeft, qreal xRight, qreal yTop, qreal yBottom, QObject *parent)
+Document::Document(const QString& name, qreal left, qreal right, qreal top, qreal bottom, QObject *parent)
         : QObject(parent), d( new DocumentPrivate())
 {
     d->_name = name;
-    d->_xLeft = xLeft;
-    d->_xRight = xRight;
-    d->_yTop = yTop;
-    d->_yBottom = yBottom;
+    d->_left = left;
+    d->_right = right;
+    d->_top = top;
+    d->_bottom = bottom;
     d->_minWidth = 0;
     d->_minHeight = 0;
     d->_saved = false;
     d->_engineBackend = new QtScriptBackend(this);
     d->_dataStructureType = DataStructurePluginManager::self()->actualPlugin();
     d->_modified = false;
+    
+    qDebug() << "------=======------======";
+    qDebug() << " Document Constructor ";
+    qDebug() << d->_dataStructureType->name();
+    qDebug() << "------=======------======";
 }
 
 void Document::setModified(const bool mod)
@@ -84,10 +91,10 @@ Document::Document(const Document& gd)
 {
 //     QObject::setParent(gd.parent());
     d->_name = gd.name();
-    d->_xLeft = gd.xLeft();
-    d->_xRight = gd.xRight();
-    d->_yTop = gd.yTop();
-    d->_yBottom = gd.yBottom();
+    d->_left = gd.left();
+    d->_right = gd.right();
+    d->_top = gd.top();
+    d->_bottom = gd.bottom();
     d->_dataStructureType = DataStructurePluginManager::self()->actualPlugin();
     d->_engineBackend = new QtScriptBackend(this);
 
@@ -102,15 +109,12 @@ Document::Document(const Document& gd)
 
 // Default Destructor
 Document::~Document() {
-
     for(int i = 0; i < d->_dataStructures.size(); i ++){
-	DataStructure *g = d->_dataStructures.at(i);
-        delete g;
+        d->_dataStructures.clear();
     }
-    delete d;
 }
 
-QList<DataStructure*>& Document::dataStructures() const { return d->_dataStructures; }
+QList< DataStructurePtr >& Document::dataStructures() const { return d->_dataStructures; }
 
 QtScriptBackend * Document::engineBackend() const{
     return d->_engineBackend;
@@ -130,65 +134,65 @@ QString Document::name() const {
 
 // gets the wheight of the drawable area
 qreal Document::height() const {
-    return d->_yBottom - d->_yTop;
+    return d->_bottom - d->_top;
 }
 
 // sets the width of the drawable area
 qreal Document::width() const {
-    return d->_xRight - d->_xLeft;
+    return d->_right - d->_left;
 }
 
-qreal Document::xLeft() const {
-    return d->_xLeft;
+qreal Document::left() const {
+    return d->_left;
 }
 
-qreal Document::xRight() const {
-    return d->_xRight;
+qreal Document::right() const {
+    return d->_right;
 }
 
-qreal Document::yTop() const {
-    return d->_yTop;
+qreal Document::top() const {
+    return d->_top;
 }
 
-qreal Document::yBottom() const {
-    return d->_yBottom;
+qreal Document::bottom() const {
+    return d->_bottom;
 }
 
-void Document::setXLeft(qreal xLeftValue)
+void Document::setLeft(qreal leftValue)
 {
-    d->_xLeft = xLeftValue;
+    d->_left = leftValue;
     //d->_modified = true;
 }
 
-void Document::setXRight(qreal xRightValue)
+void Document::setRight(qreal rightValue)
 {
-    d->_xRight = xRightValue;
+    d->_right = rightValue;
     //d->_modified = true;
 }
 
-void Document::setYTop(qreal yTopValue)
+void Document::setTop(qreal topValue)
 {
-    d->_yTop = yTopValue;
+    d->_top = topValue;
     //d->_modified = true;
 }
 
-void Document::setYBottom(qreal yBottomValue)
+void Document::setBottom(qreal bottomValue)
 {
-    d->_yBottom = yBottomValue;
+    d->_bottom = bottomValue;
     //d->_modified = true;
 }
 
 QRectF Document::size()
 {
-    return QRectF(d->_xLeft, d->_yTop,d->_xRight - d->_xLeft, d->_yBottom - d->_yTop );
+    return QRectF(d->_left, d->_top,d->_right - d->_left, d->_bottom - d->_top );
 }
 
 bool Document::isPointAtDocument(qreal x, qreal y)  const {
 
-    if (x < d->_xLeft)      return false;
-    if (x > d->_xRight)     return false;
-    if (y < d->_yTop)       return false;
-    if (y > d->_yBottom)    return false;
+    if (x < d->_left)      return false;
+    if (x > d->_right)     return false;
+    if (y < d->_top)       return false;
+    if (y > d->_bottom)    return false;
 
     return true;
 }
@@ -197,18 +201,18 @@ void Document::changeMinimalSize(qreal width, qreal height) {
     if (width>=0) d->_minWidth = width;
     if (height>=0) d->_minHeight = height;
 
-    if (width > d->_xRight - d->_xLeft) {
-        d->_xLeft -= (d->_xRight- d->_xLeft - width)/2;
-        d->_xRight += (d->_xRight- d->_xLeft - width)/2;
+    if (width > d->_right - d->_left) {
+        d->_left -= (d->_right- d->_left - width)/2;
+        d->_right += (d->_right- d->_left - width)/2;
         emit resized();
     }
     else {
         resizeDocumentBorder(BorderLeft);
     }
 
-    if (height > d->_yBottom - d->_yTop) {
-        d->_yTop -= (d->_yBottom - d->_yTop - height)/2;
-        d->_yBottom += (d->_yBottom - d->_yTop - height)/2;
+    if (height > d->_bottom - d->_top) {
+        d->_top -= (d->_bottom - d->_top - height)/2;
+        d->_bottom += (d->_bottom - d->_top - height)/2;
         emit resized();
     }
     else {
@@ -221,21 +225,21 @@ void Document::resizeDocumentIncrease()
     int elements = dataStructures().size();
     for (int i = 0; i < elements; i++) {
         bool resizeDocument = false;
-        foreach( Data* n,  dataStructures().at(i)->dataList() ){
-            if (n->x() < d->_xLeft+GraphScene::kBORDER) {
-                setXLeft(d->_xLeft-GraphScene::kBORDER);
+        foreach( DataPtr n,  dataStructures().at(i)->dataList() ){
+            if (n->x() < d->_left+GraphScene::kBORDER) {
+                setLeft(d->_left-GraphScene::kBORDER);
                 resizeDocument = true;
             }
-            if (n->x() > d->_xRight-GraphScene::kBORDER) {
-                setXRight(d->_xRight+GraphScene::kBORDER);
+            if (n->x() > d->_right-GraphScene::kBORDER) {
+                setRight(d->_right+GraphScene::kBORDER);
                 resizeDocument = true;
             }
-            if (n->y() < d->_yTop+GraphScene::kBORDER) {
-                setYTop(d->_yTop-GraphScene::kBORDER);
+            if (n->y() < d->_top+GraphScene::kBORDER) {
+                setTop(d->_top-GraphScene::kBORDER);
                 resizeDocument = true;
             }
-            if (n->y() > d->_yBottom-GraphScene::kBORDER) {
-                setYBottom(d->_yBottom+GraphScene::kBORDER);
+            if (n->y() > d->_bottom-GraphScene::kBORDER) {
+                setBottom(d->_bottom+GraphScene::kBORDER);
                 resizeDocument = true;
             }
         }
@@ -251,22 +255,22 @@ void Document::resizeDocumentBorder(Document::Border orientation) {
 
     // scans doubled border of specified size: if empty or not
     for (int i = 0; i < elements; i++) {
-    foreach( Data* n,  dataStructures().at(i)->dataList() ){
+    foreach( DataPtr n,  dataStructures().at(i)->dataList() ){
         switch (orientation) {
         case BorderLeft: {
-            if (n!=0 && n->x() < d->_xLeft+GraphScene::kBORDER*2) empty=false;
+            if (n!=0 && n->x() < d->_left+GraphScene::kBORDER*2) empty=false;
             break;
         }
         case BorderRight: {
-            if (n!=0 && n->x() > d->_xRight-GraphScene::kBORDER*2) empty=false;
+            if (n!=0 && n->x() > d->_right-GraphScene::kBORDER*2) empty=false;
             break;
         }
         case BorderTop: {
-            if (n!=0 && n->y() < d->_yTop+GraphScene::kBORDER*2) empty=false;
+            if (n!=0 && n->y() < d->_top+GraphScene::kBORDER*2) empty=false;
             break;
         }
         case BorderBottom: {
-            if (n!=0 && n->y() > d->_yBottom-GraphScene::kBORDER*2) empty=false;
+            if (n!=0 && n->y() > d->_bottom-GraphScene::kBORDER*2) empty=false;
             break;
         }
         }
@@ -277,30 +281,30 @@ void Document::resizeDocumentBorder(Document::Border orientation) {
     if (empty==true) {
         switch (orientation) {
         case BorderLeft: {
-            if (d->_xRight-d->_xLeft < GraphScene::kBORDER*4)
+            if (d->_right-d->_left < GraphScene::kBORDER*4)
                 return;
-            setXLeft(d->_xLeft+GraphScene::kBORDER);
+            setLeft(d->_left+GraphScene::kBORDER);
             emit resized();
             break;
         }
         case BorderRight: {
-            if (d->_xRight-d->_xLeft < GraphScene::kBORDER*4)
+            if (d->_right-d->_left < GraphScene::kBORDER*4)
                 return;
-            setXRight(d->_xRight-GraphScene::kBORDER);
+            setRight(d->_right-GraphScene::kBORDER);
             emit resized();
             break;
         }
         case BorderTop: {
-            if (d->_yBottom-d->_yTop < GraphScene::kBORDER*4)
+            if (d->_bottom-d->_top < GraphScene::kBORDER*4)
                 return;
-            setYTop(d->_yTop+GraphScene::kBORDER);
+            setTop(d->_top+GraphScene::kBORDER);
             emit resized();
             break;
         }
         case BorderBottom: {
-            if (d->_yBottom-d->_yTop < GraphScene::kBORDER*4)
+            if (d->_bottom-d->_top < GraphScene::kBORDER*4)
                 return;
-            setYBottom(d->_yBottom-GraphScene::kBORDER);
+            setBottom(d->_bottom-GraphScene::kBORDER);
             emit resized();
             break;
         }
@@ -313,17 +317,17 @@ bool Document::isPointAtDocument(QPointF point)  const {
 }
 
 bool Document::isModified() const{
-  return d->_modified;
+    return d->_modified;
 }
 
 void Document::cleanUpBeforeConvert()
 {
-  foreach (DataStructure * ds, d->_dataStructures)
-    ds->cleanUpBeforeConvert();
+    foreach (DataStructurePtr ds, d->_dataStructures)
+        ds->cleanUpBeforeConvert();
 }
 
 
-void Document::setActiveDataStructure(DataStructure *g){
+void Document::setActiveDataStructure(DataStructurePtr g){
     if (d->_dataStructures.indexOf(g) != -1){
         d->_activeDataStructure = g;
         emit activeDataStructureChanged(g);
@@ -331,14 +335,14 @@ void Document::setActiveDataStructure(DataStructure *g){
     }
 }
 
-DataStructure* Document::addDataStructure(QString name) {
-    DataStructure *g = DataStructurePluginManager::self()->createNewDataStructure(this,
+DataStructurePtr Document::addDataStructure(QString name) {
+    DataStructurePtr g = DataStructurePluginManager::self()->createNewDataStructure(this,
                                                                                   d->_dataStructureType->name());
     g->setName(name);
     d->_dataStructures.append(g);
     d->_activeDataStructure = g;
     d->_modified = true;
-    connect(g, SIGNAL(changed()), this, SLOT(setModified()));
+    connect(g.get(), SIGNAL(changed()), this, SLOT(setModified()));
     emit dataStructureCreated(g);
     return g;
 }
@@ -351,12 +355,85 @@ const QString& Document::documentPath() const {
     return d->_lastSavedDocumentPath;
 }
 
-void Document::remove(DataStructure *dataStructure){
+void Document::remove(DataStructurePtr dataStructure){
     d->_dataStructures.removeOne(dataStructure);
+    qDebug() << "Document::remove(Datastructure)";
     d->_modified = true;
 }
 
-bool Document::saveAsInternalFormat(const QString& filename) {
+/**
+ * A Document Internal format may look like this:
+ * 
+ * #                                '#'s are comments.
+ *                              
+ * [Document Properties]            # Canvas Size and Data Structure initialization.
+ * Width  : [integer]               
+ * Height : [integer]               
+ * DataStructurePlugin : [string]   
+ * 
+ * # Data Structre Definitions.
+ * # Data Structures are layered, so there can be N of them.
+ * # Every Data and Pointer below a declaration of a DataStructure
+ * # belongs to that data structure.
+ * 
+ * [DataStructure Name1] 
+ * 
+ * DataColor    : [RGB in hex]  # Color of newly created Datas ( Nodes )
+ * Pointercolor : [RBG in Hex]  # Color of newly created Datas ( Nodes )
+ * name         : [string]      # Name of the data structure acessible in the scripting interface.
+ * 
+ * visibility : [bool]           # is this DataStructure visible?
+ * 
+ * ShowNamesInData      : [bool] # should the canvas show the name of the data on the screen?
+ * ShowNamesInNPointers : [bool] # ↑
+ * ShowValuesInData     : [bool] # ↑
+ * ShowValuesInPointers : [bool] # ↑
+ * 
+ * PluginDefinedProperty1 : propertyValue; # the plugins can define other properties for the data structure.
+ * PluginDefinedProperty1 : propertyValue;
+ * PluginDefinedProperty1 : propertyValue;
+ * 
+ * UserDefinedProperty1 : propertyValue1 # the user can define other properties for the data structure.
+ * UserDefinedProperty2 : propertyValue2
+ * UserDefinedProperty3 : propertyValue3
+ * 
+ * [Data 1] 
+ * x     : [real]
+ * y     : [real]
+ * color : [string in "0xFFFFFF" format]
+ * name  : [string]
+ * value : [string]
+ * size  : [float]
+ * 
+ * property1 : propertyValue1
+ * property2 : propertyValue2
+ * property3 : propertyValue3
+ *
+ * [Data 2] <- same as above.
+ * x     : [integer]
+ * y     : [integer]
+ * color : [string in "0xFFFFFF" format]
+ * name  : [string]
+ * value : [string]
+ * size  : [float]
+ * 
+ * property1 : propertyValue1
+ * property2 : propertyValue2
+ * property3 : propertyValue3
+ * 
+ * [Pointer 1 -> 2]
+ * name     : [string]
+ * value    : [string]
+ * style    : [string]
+ * color    : [string in "0xFFFFFF" format]
+ * width    : [float]
+ * 
+ * property1 : propertyValue1
+ * property2 : propertyValue2
+ * property3 : propertyValue3
+ */
+
+ bool Document::saveAsInternalFormat(const QString& filename) {
     d->_buf.clear();
 
     KSaveFile saveFile( !filename.endsWith(".graph") ? QString("%1.graph").arg(filename) : filename);
@@ -368,25 +445,33 @@ bool Document::saveAsInternalFormat(const QString& filename) {
 
     QTextStream stream(&saveFile);
     stream.setCodec("UTF-8");
-
+    
+     d->_buf = QString("[Document Properties] \n")
+             % QString("top : ") % QString::number(top()) % QChar('\n')
+             % QString("bottom : ") % QString::number(bottom()) % QChar('\n')
+             % QString("left : ") % QString::number(left()) % QChar('\n')
+             % QString("right : ") % QString::number(right()) % QChar('\n')
+             % QString("DataStructurePlugin : ") % DataStructurePluginManager::self()->actualPlugin()->name() % QChar('\n')
+             % QChar('\n');
+            
     for (int i = 0; i < d->_dataStructures.count(); i++) {
-        DataStructure *g = d->_dataStructures.at(i);
+        DataStructurePtr g = d->_dataStructures.at(i);
 
         d->_buf += QString("[DataStructure %1] \n").arg(i).toUtf8();
 
-        savePropertiesInternalFormat(g);
+        savePropertiesInternalFormat(g.get());
 
-        foreach( Data *n, g->dataList()) {
+        foreach( DataPtr n, g->dataList()) {
             d->_buf += QString("[Data %1]\n").arg(g->dataList().indexOf(n)).toUtf8();
-            savePropertiesInternalFormat(n);
+            savePropertiesInternalFormat(n.get());
         }
 
         int from, to;
-        foreach( Pointer *e, g->pointers()) {
+        foreach( PointerPtr e, g->pointers()) {
             from = g->dataList().indexOf(e->from());
             to = g->dataList().indexOf(e->to());
             d->_buf += QString("[Pointer %1->%2]\n").arg(from).arg(to).toUtf8();
-            savePropertiesInternalFormat(e);
+            savePropertiesInternalFormat(e.get());
         }
 
         /*     buf += " \n \n ############ GROUPS ########### \n \n";
@@ -442,14 +527,12 @@ void Document::savePropertiesInternalFormat(QObject *o) {
 void Document::loadFromInternalFormat(const QString& filename) {
     QFile f(filename);
     if ( !f.open(QIODevice::ReadOnly | QIODevice::Text) ) {
-        kDebug() << "File not open " << filename.toUtf8();
+        qDebug() << "File not open " << filename.toUtf8();
         return;
     }
-
-    DataStructure* tmpDataStructure = 0;
-   //DataStructureGroup *tmpGroup = 0;
+    DataStructurePtr tmpDataStructure;
     QObject *tmpObject = 0;
-
+    DataPtr tmpDataPtr;
 
     QTextStream in(&f);
     in.setCodec("UTF-8");
@@ -460,22 +543,51 @@ void Document::loadFromInternalFormat(const QString& filename) {
         if (str.startsWith('#')) { //! Ignore it, commented line.
             continue;
         }
-
+        else if(str.startsWith("[Document Properties]")){
+            
+            QString dataLine = in.readLine().simplified();
+            while (!in.atEnd() && !dataLine.isEmpty()) {
+                /**/ if (dataLine.startsWith("top :"))      setTop(dataLine.section(' ',2).toInt());
+                else if (dataLine.startsWith("bottom :"))   setBottom(dataLine.section(' ',2).toInt());
+                else if (dataLine.startsWith("left :"))     setLeft(dataLine.section(' ',2).toInt());
+                else if (dataLine.startsWith("right :"))    setRight(dataLine.section(' ',2).toInt());
+                
+           // TODO: Wagner, How this thing works?
+                else if (dataLine.startsWith("DataStructurePlugin :")){
+                    DataStructurePluginManager::self()->setDataStructurePlugin(dataLine.section(' ',2));
+                    d->_dataStructureType =  DataStructurePluginManager::self()->actualPlugin();
+                }
+                else if ( !dataLine.isEmpty())               break; // go to the last if and finish populating.
+                dataLine = in.readLine().simplified();    
+            }
+            tmpObject = this;
+        }
         else if (str.startsWith("[DataStructure")) {
             QString gName = str.section(' ',1,1);
             gName.remove(']');
             tmpDataStructure = DataStructurePluginManager::self()->createNewDataStructure(this);
             tmpDataStructure->setName(gName.toAscii());
-            tmpObject = tmpDataStructure;
             d->_dataStructures.append(tmpDataStructure);
-            kDebug() << "DataStructure Created";
+            tmpObject = tmpDataStructure.get();
         }
 
         else if (str.startsWith("[Data")) {
-            QString nName = str.section(' ',1,1);
-            nName.remove(']');
-            tmpObject = tmpDataStructure->addData(nName);
-            kDebug() << "Data Created";
+            tmpDataPtr = tmpDataStructure->addData(QString());
+            QString dataLine = in.readLine().simplified();
+            qreal posX = 0;
+            qreal posY = 0;
+            while (!in.atEnd() && !dataLine.isEmpty()) {
+                /**/ if (dataLine.startsWith("x :"))         posX = dataLine.section(' ',2).toFloat();
+                else if (dataLine.startsWith("y :"))         posY = dataLine.section(' ',2).toFloat();
+                else if (dataLine.startsWith("value :"))     tmpDataPtr->setValue(dataLine.section(' ',2).toInt());
+                else if (dataLine.startsWith("color :"))     tmpDataPtr->setColor(dataLine.section(' ',2));
+                else if (dataLine.startsWith("name :"))      tmpDataPtr->setName(dataLine.section(' ', 2));
+                else if ( !dataLine.isEmpty())               break; // go to the last if and finish populating.
+                dataLine = in.readLine().simplified();
+            }
+            tmpDataPtr->setPos(posX, posY);
+            qDebug() << tmpDataPtr->x() << tmpDataPtr->y();
+            tmpObject = tmpDataPtr.get();
         }
 
         else if (str.startsWith("[Pointer")) {
@@ -485,9 +597,21 @@ void Document::loadFromInternalFormat(const QString& filename) {
             QString nameFrom = eName.section("->", 0,0);
             QString nameTo = eName.section("->", 1,1);
 
-            tmpObject = tmpDataStructure->addPointer(tmpDataStructure->dataList().at(nameFrom.toInt()),
+            PointerPtr tmpPointer = tmpDataStructure->addPointer(tmpDataStructure->dataList().at(nameFrom.toInt()),
                                                      tmpDataStructure->dataList().at(nameTo.toInt()));
-            kDebug() << "Pointer Created";
+            
+            QString dataLine = in.readLine().simplified();
+            while (!in.atEnd() && !dataLine.isEmpty()) {
+                /**/ if (dataLine.startsWith("width :"))     tmpPointer->setWidth(dataLine.section(' ',2).toInt());
+                else if (dataLine.startsWith("value :"))     tmpPointer->setValue(dataLine.section(' ',2));
+                else if (dataLine.startsWith("color :"))     tmpPointer->setColor(dataLine.section(' ',2));
+                else if (dataLine.startsWith("width :"))     tmpPointer->setColor(dataLine.section(' ',2).toFloat());
+                else if (dataLine.startsWith("style :"))     tmpPointer->setColor(dataLine.section(' ',2));
+                
+                else if ( !dataLine.isEmpty())                break; // go to the last if and finish populating.
+                dataLine = in.readLine().simplified();
+            }
+            tmpObject = tmpPointer.get();
         }
         else if (str.startsWith("[Group")) {
             /*QString gName = str.section(" ",1,1);
@@ -499,15 +623,15 @@ void Document::loadFromInternalFormat(const QString& filename) {
             QString propertyValue = str.section(':',1,1).trimmed();
             tmpObject->setProperty( propertyName.toUtf8() , propertyValue );
         }
-        else {
-//            // tmpGroup->append( tmpDataStructure->data(str));
-        }
+        //else {
+        //      tmpGroup->append( tmpDataStructure->data(str));
+        //}
     }
     d->_modified = false;
-    kDebug() << "DataStructure Document Loaded.";
+    qDebug() << "DataStructure Document Loaded.";
 }
 
-DataStructure *Document::activeDataStructure() const { return d->_activeDataStructure; }
+DataStructurePtr Document::activeDataStructure() const { return d->_activeDataStructure; }
 
 QString Document::dataStructureTypeName() const
 {
