@@ -20,31 +20,39 @@
 */
 
 #include "MainWindow.h"
+
 //Qt related includes
+#include <QtGui/QCloseEvent>
+#include <QtGui/QComboBox>
+#include <QtGui/QGraphicsView>
+#include <QtGui/QLabel>
+#include <QtGui/QLayout>
+#include <QtGui/QSplitter>
+#include <QtGui/QStackedWidget>
+#include <QtGui/QToolBar>
+#include <QtGui/QVBoxLayout>
 
 // KDE Related Includes
 #include <KActionCollection>
-#include <KIcon>
-#include <QSplitter>
-#include <KDebug>
 #include <KApplication>
+#include <KDebug>
+#include <KIcon>
+#include <KPushButton>
+#include <KStatusBar>
+#include <KTar>
 #include <KTextBrowser>
 #include <KMessageBox>
-#include <KApplication>
-#include <QGraphicsView>
 #include <klocalizedstring.h>
-#include <KStatusBar>
 #include <KConfigDialog>
-#include <KTar>
 #include <kfiledialog.h>
-#include "TabWidget.h"
 
 // UI RELATED INCLUDES
-
 #include "GraphLayers.h"
 #include "GraphVisualEditor.h"
 #include "GraphScene.h"
 #include "CodeEditor.h"
+#include "TabWidget.h"
+#include "GraphicsLayout.h"
 
 // MODEL Related Includes
 #include "model_GraphDocument.h"
@@ -57,12 +65,14 @@
 
 // Action Related Includes
 #include "AbstractAction.h"
-#include "AddData.h"
-#include "AddConnection.h"
-#include "MoveData.h"
-#include "Select.h"
-#include "DeleteAction.h"
+#include "AddDataHandAction.h"
+#include "AddConnectionHandAction.h"
+#include "SelectMoveHandAction.h"
+#include "DeleteHandAction.h"
 #include "AlignAction.h"
+#include "ZoomAction.h"
+#include "DeleteAction.h"
+
 #include <KNS3/DownloadDialog>
 #include <knewstuff3/uploaddialog.h>
 
@@ -76,7 +86,6 @@
 #include <QActionGroup>
 
 #include "../Plugins/PluginManager.h"
-#include <KPushButton>
 #include <QMutexLocker>
 #include <QScriptEngineDebugger>
 #include "IncludeManagerSettings.h"
@@ -86,10 +95,9 @@
 #include <DataStructurePluginInterface.h>
 #include <DataStructurePluginManager.h>
 #include "DocumentManager.h"
-#include <QCloseEvent>
-#include <KMessageBox>
-#include "zoom.h"
-#include "../GraphicsItem/GraphicsLayout.h"
+
+
+
 #include "PossibleIncludes.h"
 
 MainWindow::MainWindow() :  KXmlGuiWindow()
@@ -117,10 +125,10 @@ MainWindow::MainWindow() :  KXmlGuiWindow()
      * this should not be hardcoded here.
      * use KWelcomeWidget instead.
      */
-     DocumentManager::self()->loadDocument();
+    DocumentManager::self()->loadDocument();
 
     GraphicsLayout::self()->setViewStyleDataNode(Settings::dataNodeDisplay());
-    GraphicsLayout::self()->setViewStyleDataEdge(Settings::dataEdgeDisplay());
+    GraphicsLayout::self()->setViewStyleDataEdge(Settings::dataEdgeDisplay());   
 }
 
 MainWindow::~MainWindow()
@@ -128,7 +136,9 @@ MainWindow::~MainWindow()
     Settings::setVSplitterSizeTop ( _vSplitter->sizes() [0] );
     Settings::setVSplitterSizeBottom ( _vSplitter->sizes() [1] );
     Settings::setHSplitterSizeLeft ( _hSplitter->sizes() [0] );
-    Settings::setHSplitterSizeRight ( _hSplitter->sizes() [1] );
+    Settings::setHSplitterSizeRight ( _hSplitter->sizes() [1] );  
+    Settings::setHScriptSplitterSizeLeft ( _hScriptSplitter->sizes() [0] );
+    Settings::setHScriptSplitterSizeRight ( _hScriptSplitter->sizes() [1] );
 
     Settings::self()->writeConfig();
 }
@@ -147,17 +157,30 @@ void MainWindow::closeEvent(QCloseEvent *event)
 
 void MainWindow::setupWidgets()
 {
+    // splits the main window horizontal
+    _vSplitter = new QSplitter ( this );
+    _vSplitter->setOrientation ( Qt::Vertical );
+
+    // setup upper half
+    QWidget *leftPanel  = setupWhiteboardPanel();         // graph properties
+    _graphVisualEditor = GraphVisualEditor::self(); // graph editor whiteboard
+    
     _hSplitter = new QSplitter ( this );
-
-    QWidget *rightPanel = setupRightPanel();
-    QWidget *leftPanel  = setupLeftPanel();
-
-
-    _hSplitter->addWidget ( leftPanel );
-    _hSplitter->addWidget ( rightPanel );
+    _hSplitter->setOrientation( Qt::Horizontal );
+    _hSplitter->addWidget (leftPanel);
+    _hSplitter->addWidget (_graphVisualEditor);
+    
+    // setup lower half
+    QWidget *scriptPanel = setupScriptPanel();
+  
+    _vSplitter->addWidget ( _hSplitter );
+    _vSplitter->addWidget ( scriptPanel );
+    
+    _hScriptSplitter->setSizes ( QList<int>() << Settings::hScriptSplitterSizeLeft() << Settings::hScriptSplitterSizeRight() << 80 );
+    _vSplitter->setSizes ( QList<int>() << Settings::vSplitterSizeTop() << Settings::vSplitterSizeBottom() );
     _hSplitter->setSizes ( QList<int>() << Settings::hSplitterSizeLeft() << Settings::hSplitterSizeRight() );
 
-    setCentralWidget ( _hSplitter );
+    setCentralWidget ( _vSplitter );
 }
 
 void MainWindow::downloadNewExamples(){
@@ -201,36 +224,58 @@ void MainWindow::uploadScript()
 }
 
 
-QWidget* MainWindow::setupRightPanel()
-{
-    _graphVisualEditor = GraphVisualEditor::self();
-
+QWidget* MainWindow::setupScriptPanel()
+{     
+    _hScriptSplitter = new QSplitter( this );
+    _hScriptSplitter->setOrientation( Qt::Horizontal );
+  
     _codeEditor = new CodeEditor ( this );
     _txtDebug = new KTextBrowser ( this );
     _txtOutput = new KTextBrowser( this );
 
-    _bottomTabs = new TabWidget ( TabWidget::TabOnBottom, this );
-    _bottomTabs->addWidget ( _codeEditor,  i18n ( "Editor" ), KIcon ( "accessories-text-editor" ) );
-    _bottomTabs->addWidget ( _txtDebug, i18n ( "Debugger" ), KIcon ( "tools-report-bug" ) );
-    _bottomTabs->addWidget ( _txtOutput, i18n("Output"), KIcon ("tools-document"));
+    QStackedWidget *stackedListing = new QStackedWidget;
+    stackedListing->addWidget(_txtOutput);
+    stackedListing->addWidget(_txtDebug);
 
+    _selectListing = new QComboBox;
+    _selectListing->addItem( KIcon ( "accessories-text-editor" ), i18n("Program Messages"));
+    _selectListing->addItem( KIcon ( "tools-report-bug" ), i18n("Debug Messages"));
+        
+    QWidget *header = new QWidget( this );
+    header->setLayout( new QHBoxLayout );
+    header->layout()->addWidget(new QLabel(i18n("Select output:")));
+    header->layout()->addWidget(_selectListing);
+    
+    QWidget *listingWidget = new QWidget(this);
+    listingWidget->setLayout( new QVBoxLayout );
+    listingWidget->layout()->addWidget( header );
+    listingWidget->layout()->addWidget( stackedListing );
+    
+    QToolBar *executeCommands = new QToolBar;
+    executeCommands->setToolButtonStyle( Qt::ToolButtonTextUnderIcon );
+    executeCommands->setOrientation(Qt::Vertical);
     _runScript = new KAction ( KIcon ( "system-run" ), i18n ( "Run" ), this );
-    connect ( _runScript, SIGNAL ( triggered() ), this, SLOT ( executeScript() ) );
-    _bottomTabs->addAction ( _runScript );
-
-    _vSplitter = new QSplitter ( this );
-    _vSplitter->setOrientation ( Qt::Vertical );
-
-    // graph editor whiteboard
-    _vSplitter->addWidget (_graphVisualEditor);
-
-    _vSplitter->addWidget ( _bottomTabs );
-    _vSplitter->setSizes ( QList<int>() << Settings::vSplitterSizeTop() << Settings::vSplitterSizeBottom() );
-
-    return _vSplitter;
+    _stopScript = new KAction ( KIcon ( "process-stop" ), i18n ( "Stop" ), this );
+    _stopScript->setEnabled(false);
+    executeCommands->addAction(_runScript);
+    executeCommands->addAction(_stopScript);
+    
+    connect(_runScript, SIGNAL(triggered()), this, SLOT(executeScript()));
+    connect(_stopScript, SIGNAL(triggered()), this, SLOT(stopScript()));
+    connect(_selectListing, SIGNAL(currentIndexChanged(int)), stackedListing, SLOT(setCurrentIndex(int)));
+    
+    _hScriptSplitter->addWidget( _codeEditor );
+    _hScriptSplitter->addWidget( listingWidget );
+    
+    QWidget *scriptInterface = new QWidget( this );
+    scriptInterface->setLayout( new QHBoxLayout );
+    scriptInterface->layout()->addWidget( _hScriptSplitter );
+    scriptInterface->layout()->addWidget( executeCommands );
+    
+    return scriptInterface;
 }
 
-QWidget* MainWindow::setupLeftPanel()
+QWidget* MainWindow::setupWhiteboardPanel()
 {
     _GraphLayers = new GraphLayers ( this );
     return _GraphLayers;
@@ -244,47 +289,64 @@ void MainWindow::setupActions()
 
     GraphScene *gc = _graphVisualEditor->scene();
 
-    _moveDataAction = new MoveDataAction ( gc, this );
+    _selectMoveAction = new SelectMoveHandAction (gc, this);
+    AddDataHandAction* addDataAction = new AddDataHandAction(gc, this );
+    AddConnectionHandAction* addConnectionAction = new AddConnectionHandAction(gc, this);
+    DeleteHandAction* deleteAction = new DeleteHandAction(gc, this);
+    ZoomAction* zoomAction = new ZoomAction(gc, this);
 
-    KActionCollection *ac = actionCollection();
+    connect(_selectMoveAction, SIGNAL(triggered()), _selectMoveAction, SLOT( sendExecuteBit() ));
+    connect(addDataAction, SIGNAL(triggered()), addDataAction, SLOT( sendExecuteBit() ));
+    connect(addConnectionAction, SIGNAL(triggered()), addConnectionAction, SLOT( sendExecuteBit() ));
+    connect(deleteAction, SIGNAL(triggered()), deleteAction, SLOT( sendExecuteBit() ));
+    connect(zoomAction, SIGNAL(triggered()), zoomAction, SLOT( sendExecuteBit() ));
+
+    _paletteActions = actionCollection();
     QActionGroup *g = new QActionGroup ( this );
 
-    g->addAction ( ac->addAction ( "move_node", _moveDataAction ) );
-    g->addAction ( ac->addAction ( "add_node", new AddNodeAction ( gc, this ) ) );
-    g->addAction ( ac->addAction ( "add_edge", new AddConnectionAction ( gc, this ) ) );
-    g->addAction ( ac->addAction ( "select", new SelectAction ( gc, this ) ) );
-    g->addAction ( ac->addAction ( "delete", new DeleteAction ( gc, this ) ) );
-    g->addAction ( ac->addAction ( "zoom", new ZoomAction ( gc, this ) ) );
-    actionCollection()->action ( "move_node" )->toggle();
-    gc->setAction ( _moveDataAction );
+    g->addAction ( _paletteActions->addAction ( "selectmove", _selectMoveAction ) );
+    g->addAction ( _paletteActions->addAction ( "add_node", addDataAction ) );
+    g->addAction ( _paletteActions->addAction ( "add_edge", addConnectionAction ) );
+    g->addAction ( _paletteActions->addAction ( "delete", deleteAction ) );
+    g->addAction ( _paletteActions->addAction ( "zoom", zoomAction ) );
+    actionCollection()->action ( "selectmove" )->toggle();
+    gc->setAction ( _selectMoveAction );
 
-    ac->addAction ( "align-hbottom",new AlignAction ( i18n ( "Align on the base" ),  AlignAction::Bottom, _graphVisualEditor ) );
-    ac->addAction ( "align-hcenter",new AlignAction ( i18n ( "Align on the center" ),AlignAction::HCenter,_graphVisualEditor ) );
-    ac->addAction ( "align-htop",   new AlignAction ( i18n ( "Align on the top" ),   AlignAction::Top,    _graphVisualEditor ) );
-    ac->addAction ( "align-vleft",  new AlignAction ( i18n ( "Align on the left" ),  AlignAction::Left,   _graphVisualEditor ) );
-    ac->addAction ( "align-vcenter",new AlignAction ( i18n ( "Align on the center" ),AlignAction::VCenter,_graphVisualEditor ) );
-    ac->addAction ( "align-vright", new AlignAction ( i18n ( "Align on the right" ), AlignAction::Right,  _graphVisualEditor ) );
+    _paletteActions->addAction ( "align-hbottom",new AlignAction ( i18nc( "Alignment", "Base" ),  AlignAction::Bottom, gc, _graphVisualEditor ) );
+    _paletteActions->addAction ( "align-hcenter",new AlignAction ( i18nc( "Alignment", "Center" ),AlignAction::HCenter,gc,_graphVisualEditor ) );
+    _paletteActions->addAction ( "align-htop",   new AlignAction ( i18nc( "Alignment", "Top" ),   AlignAction::Top,    gc,_graphVisualEditor ) );
+    _paletteActions->addAction ( "align-vleft",  new AlignAction ( i18nc( "Alignment", "Left" ),  AlignAction::Left,   gc,_graphVisualEditor ) );
+    _paletteActions->addAction ( "align-vcenter",new AlignAction ( i18nc( "Alignment", "Center" ),AlignAction::VCenter,gc,_graphVisualEditor ) );
+    _paletteActions->addAction ( "align-vright", new AlignAction ( i18nc( "Alignment", "Right" ), AlignAction::Right,  gc,_graphVisualEditor ) );
+    _paletteActions->addAction ( "align-circle", new AlignAction ( i18nc( "Alignment", "Circle" ),  AlignAction::Circle,  gc,_graphVisualEditor ) );
+    _paletteActions->addAction ( "align-tree", new AlignAction ( i18nc( "Alignment", "Minimize Crossing Edges" ),  AlignAction::MinCutTree, gc, _graphVisualEditor ) );
 
-    createAction("document-new",     i18n("New Graph"),         "new-graph",         Qt::Key_N, SLOT(newGraph()),    this);
-    createAction("document-open",    i18n("Open Graph"),        "open-graph",        Qt::Key_0, SLOT(openGraph()),   this);
-    createAction("document-save",    i18n("Save Graph"),        "save-graph",        Qt::Key_S, SLOT(saveGraph()),   this);
-    createAction("document-save-as", i18n("Save Graph as"),     "save-graph-as",     Qt::Key_W, SLOT(saveGraphAs()), this);
-    createAction("",                 i18n("Download Examples"), "download", Qt::Key_D, SLOT(downloadNewExamples()),  this);
-    createAction("",                 i18n("Upload script"),     "upload",   Qt::Key_U, SLOT(uploadScript()),  this);
+    // Menu actions
+    createAction("document-new",     i18n("New Graph"),         "new-graph",         SLOT(newGraph()),    this);
+    createAction("document-open",    i18n("Open Graph"),        "open-graph",        SLOT(openGraph()),   this);
+    createAction("document-save",    i18n("Save Graph"),        "save-graph",        SLOT(saveGraph()),   this);
+    createAction("document-save-as", i18n("Save Graph as"),     "save-graph-as",     SLOT(saveGraphAs()), this);
+    createAction("",                 i18n("Download Examples"), "download",          SLOT(downloadNewExamples()),  this);
+    createAction("",                 i18n("Upload script"),     "upload",            SLOT(uploadScript()),  this);
+    createAction("document-save",    i18n("Save All"),          "save-all",        Qt::Key_S, SLOT(saveAll()),   this);
+    
+    createAction("document-save-as", i18n("Possible Includes"), "possible_includes", SLOT(showPossibleIncludes()), this);
 
-    createAction("document-save-as", i18n("Possible Includes"), "possible_includes", Qt::Key_P, SLOT(showPossibleIncludes()), this);
+    createAction("document-new",     i18n("New Script"),        "new-script",        SLOT(newScript()),    _codeEditor);
+    createAction("document-open",    i18n("Open Script"),       "open-script",       SLOT(openScript()),   _codeEditor);
+    createAction("document-save",    i18n("Save Script"),       "save-script",       SLOT(saveActiveScript()),   _codeEditor);
+    createAction("document-save-as", i18n("Save Script as"),    "save-script-as",    SLOT(saveActiveScriptAs()), _codeEditor);
 
-    createAction("document-new",     i18n("New Script"),        "new-script",        Qt::Key_N, SLOT(newScript()),    _codeEditor);
-    createAction("document-open",    i18n("Open Script"),       "open-script",       Qt::Key_O, SLOT(openScript()),   _codeEditor);
-    createAction("document-save",    i18n("Save Script"),       "save-script",       Qt::Key_S, SLOT(saveScript()),   _codeEditor);
-    createAction("document-save-as", i18n("Save Script as"),    "save-script-as",    Qt::Key_W, SLOT(saveScriptAs()), _codeEditor);
-
-    kDebug() << "Creating Actions (import export)";
-    if (PluginManager::instance()->filePlugins().count()) {
-        createAction("document-save-as", i18n("Import"), "import", Qt::Key_I, SLOT(importFile()), this);
-        createAction("document-save-as", i18n("Export"), "export", Qt::Key_E, SLOT(exportFile()), this);
+    // eventually create hooks for file plugins
+    PluginManager::instance()->loadFilePlugins();
+    if (PluginManager::instance()->filePlugins().count()>0) {
+        createAction("document-open", i18n("Import Graph"), "import-graph", SLOT(importFile()), this);
+        createAction("document-save", i18n("Export Graph"), "export-graph", SLOT(exportFile()), this);
     }
 
+    // EDIT actions
+    actionCollection()->addAction("delete-selected", new DeleteAction( i18n("Delete"), _graphVisualEditor->scene(), 0) );
+    
     KStandardAction::quit ( kapp, SLOT ( quit() ),  actionCollection() );
 }
 
@@ -295,9 +357,19 @@ void MainWindow::createAction(const QByteArray& iconName, const QString& actionT
     action->setShortcut ( shortcut);
     action->setShortcutContext ( Qt::WidgetShortcut );
     actionCollection()->addAction ( actionName, action );
-    connect ( action, SIGNAL ( triggered ( bool ) ), parent, slot );
+    connect(action, SIGNAL(triggered(bool)), parent, slot );
 }
 
+void MainWindow::createAction(const QByteArray& iconName, const QString& actionTitle, const QString& actionName,
+                              const char* slot, QObject *parent)
+{
+    KAction* action = new KAction ( KIcon ( iconName ), actionTitle, parent );
+    action->setShortcutContext ( Qt::WidgetShortcut );
+    actionCollection()->addAction ( actionName, action );
+    connect(action, SIGNAL(triggered(bool)), parent, slot);
+}
+
+
 void MainWindow::showSettings()
 {
      KConfigDialog dialog(this,  "settings", Settings::self());
@@ -334,10 +406,10 @@ void MainWindow::setupToolsPluginsAction()
     foreach (  ToolsPluginInterface* p, avaliablePlugins ){
         action = new KAction ( p->displayName(), this );
         action->setData(count++);
-        connect ( action, SIGNAL ( triggered ( bool ) ),this, SLOT ( runToolPlugin() ) );
-        pluginList.append ( action );
+        connect(action, SIGNAL(triggered(bool)), this, SLOT(runToolPlugin()));
+        pluginList.append( action );
     }
-    plugActionList ( "tools_plugins", pluginList );
+    plugActionList( "tools_plugins", pluginList );
 }
 
 void MainWindow::setupDSPluginsAction()
@@ -345,18 +417,26 @@ void MainWindow::setupDSPluginsAction()
     QList <QAction*> pluginList;
     QAction* action = 0;
     unplugActionList ( "DS_plugins" );
+    
     QList < DataStructurePluginInterface*> avaliablePlugins = DataStructurePluginManager::self()->pluginsList();
+    
     QActionGroup * group = new QActionGroup(this);
+    
     int count = 0;
     foreach ( DataStructurePluginInterface* p, avaliablePlugins ) {
         action = new KAction ( p->name(), this );
         action->setData(count++);
         action->setCheckable(true);
-        if (p->name() == DataStructurePluginManager::self()->actualPluginName()){
+        
+        if (p->name() == DataStructurePluginManager::self()->pluginName()){
           action->setChecked(true);
         }
+        
         action->setActionGroup(group);
-        connect ( action, SIGNAL ( triggered ( bool ) ),DataStructurePluginManager::self(), SLOT ( changeActiveDataStructure()) );
+        
+        connect ( action, SIGNAL (triggered(bool)),
+                  DataStructurePluginManager::self(), SLOT(setDataStructurePlugin()));
+        
         pluginList.append ( action );
     }
     plugActionList ( "DS_plugins", pluginList );
@@ -376,7 +456,7 @@ void MainWindow::setupDocumentsList(){
           action->setChecked(true);
         }
         action->setActionGroup(group);
-        connect ( action, SIGNAL ( triggered ( bool ) ), DocumentManager::self(), SLOT ( changeDocument()) );
+        connect(action, SIGNAL(triggered(bool)), DocumentManager::self(), SLOT(changeDocument()));
         pluginList.append ( action );
     }
 
@@ -392,13 +472,14 @@ void MainWindow::setActiveDocument ( )
     _graphVisualEditor->setActiveDocument();
     _GraphLayers->setActiveDocument();
 
-    connect ( this, SIGNAL(runTool(  ToolsPluginInterface*,Document*)),
-                activeDocument->engineBackend(), SLOT (runTool( ToolsPluginInterface*,Document*)));
+    connect ( this, SIGNAL(runTool(ToolsPluginInterface*,Document*)),
+                activeDocument->engineBackend(), SLOT(runTool(ToolsPluginInterface*,Document*)));
 
 //     connect(this, SIGNAL(startEvaluation()),   engine,  SLOT(start()));
     connect( engine, SIGNAL(sendDebug(QString)), this,  SLOT(debugString(QString)));
+    connect( engine, SIGNAL(scriptError()), this, SLOT(showDebugOutput()));
     connect( engine, SIGNAL(sendOutput(QString)), this, SLOT(outputString(QString)));
-    connect( engine, SIGNAL(finished()),   _bottomTabs, SLOT(setPlayString()));
+    connect( engine, SIGNAL(finished()), this, SLOT(disableStopAction()));
     activeDocument->setModified(false);
 }
 
@@ -467,18 +548,38 @@ void MainWindow::saveGraphAs(){
     d->saveAsInternalFormat ( KFileDialog::getSaveFileName() );
 }
 
-// void MainWindow::debug ( const QString& s ){
-//     _txtDebug->insertPlainText ( s );
-// }
+void MainWindow::saveAll() {
+    saveGraph();
+    _codeEditor->saveAllScripts();
+}
+
 
 int MainWindow::saveIfChanged(){
-    if ( DocumentManager::self()->activeDocument()->isModified() ){
-        const int btnCode = KMessageBox::warningYesNoCancel ( this, i18n ( "Do you want to save your unsaved document?" ) );
+    if ( DocumentManager::self()->activeDocument()->isModified() && !_codeEditor->isModified() ){
+        const int btnCode = KMessageBox::warningYesNoCancel ( this, i18n ( 
+            "Changes on your graph document are unsaved. Do you want to save your changes?" ) );
         if ( btnCode == KMessageBox::Yes ){
             saveGraph();
         }
         return btnCode;
     }
+    if ( !DocumentManager::self()->activeDocument()->isModified() && _codeEditor->isModified() ){
+        const int btnCode = KMessageBox::warningYesNoCancel ( this, i18n ( 
+            "Changes on your script files are unsaved. Do you want to save all unsaved scripts?" ) );
+        if ( btnCode == KMessageBox::Yes ){
+            _codeEditor->saveAllScripts();
+        }
+        return btnCode;
+    }
+    if ( DocumentManager::self()->activeDocument()->isModified() && _codeEditor->isModified() ){
+        const int btnCode = KMessageBox::warningYesNoCancel ( this, i18n ( 
+            "Changes on your script files and on your graph document are unsaved. Do you want to save your graph document and all unsaved scripts?" ) );
+        if ( btnCode == KMessageBox::Yes ){
+            _codeEditor->saveAllScripts();
+            saveGraph();
+        }
+        return btnCode;
+    }
     return KMessageBox::No;
 }
 
@@ -547,7 +648,7 @@ void MainWindow::dsChanged(){
 #ifdef USING_QTSCRIPT
 
 void MainWindow::executeScript(const QString& text) {
-    kDebug() << "Going to execut the script";
+    kDebug() << "Going to execute the script";
     if (_txtDebug == 0)   return;
     if ( scene() == 0)    return;
 
@@ -561,17 +662,41 @@ void MainWindow::executeScript(const QString& text) {
                                  _codeEditor->document()->documentName());
 
     QtScriptBackend *engine = DocumentManager::self()->activeDocument()->engineBackend();
-    if ( !engine->isRunning() ){
-        _bottomTabs->setStopString();
-        engine->setScript(script, DocumentManager::self()->activeDocument());
-        engine->start();
-    }else{
-       _bottomTabs->setPlayString();
-       engine->stop();
+    if (engine->isRunning() ) {
+        engine->stop();
     }
+
+    enableStopAction();
+    engine->setScript(script, DocumentManager::self()->activeDocument());
+    engine->start();
+}
+
+void MainWindow::stopScript() {
+    kDebug() << "Going to stop the script";
+    if (_txtDebug == 0)   return;
+    if ( scene() == 0)    return;
+
+    QtScriptBackend *engine = DocumentManager::self()->activeDocument()->engineBackend();
+
+    disableStopAction();
+    engine->stop();
 }
 
 #endif
 
+
+void MainWindow::enableStopAction(){
+    _stopScript->setEnabled(true);
+}
+
+void MainWindow::disableStopAction(){
+    _stopScript->setEnabled(false);
+}
+
+void MainWindow::showDebugOutput()
+{
+    _selectListing->setCurrentIndex(1);
+}
+
 void MainWindow::outputString ( const QString& s ) { _txtOutput->append ( s ); }
 void MainWindow::debugString ( const QString& s )  { _txtDebug->append ( s );  }
